package com.dummies.silentmodetoggle;

import android.app.Activity;
import android.media.AudioManager;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.ImageView;

import com.dummies.silentmodetoggle.util.RingerHelper;

public class MainActivity extends Activity {

    AudioManager audioManager;

    /**
     * Diese Methode wird aufgerufen, um die Aktivitt zu initialisieren,
     * nachdem der Java-Konstruktor fr diese Klasse aufgerufen wurde.
     * Dies ist typisch, wenn Sie setContentView aufrufen, um Ihr Layout
     * zu erweitern, und mit findViewById Ihre Ansichten initialisieren.
     * @param savedInstanceState enthlt zustzliche Daten ber den
     *                           gespeicherten Status der Aktivitt,
     *                           wenn sie zuvor geschlossen und jetzt
     *                           aus dem gespeicherten Status hergestellt wird.
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        // Immer zuerst super.onCreate() aufrufen.
        super.onCreate(savedInstanceState);

        // Referenz auf den AudioManager von Android aktivieren, sodass
        // wir ihn zum Umschalten unserer Klingel verwenden knnen.
        audioManager = (AudioManager) getSystemService(AUDIO_SERVICE);

        // Unser Layout mit der Layoutdatei res/layout/activity_main.xml
        // initialisierne, die unsere Ansichten fr diese Aktivitt enthlt.
        setContentView(R.layout.activity_main);

        // Die Ansicht "content" in unserer Layoutdatei finden.
        FrameLayout contentView =
                (FrameLayout) findViewById(R.id.content);

        // Einen Click-Listener fr die contentView setzen, der den Klingelstatus
        // des Telefons umschaltet, und dann die UI aktualisieren, um den neuen
        // Status anzuzeigen.
        // Der aufmerksame Leser fragt sich jetzt vielleicht, warum wir keinen
        // Click-Listener auf das eigentliche Bild, sondern auf den Rahmen
        // gesetzt haben. Dafr gibt es zwei Grnde:
        //   1. Wir wollen, dass die ganze Seite angeklickt werden kann,
        //      und nicht nur das Bild.
        //   2. Wir verwenden android:foreground mit dem Wert
        //      ?android:attr/selectableItemBackground, um einen hbschen
        //      Ripple-Effekt zu schaffen, wenn wir auf die Seite klicken. Aus
        //      technischen Grnden ist es nicht mglich, dies auf eine
        //      ImageView anzuwenden.
        contentView.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {

                // Klingelmodus umschalten. Normal --> Stumm,
                // Stumm --> Normal.
                RingerHelper.performToggle(audioManager);

                // UI aktualisieren, damit sie den neuen Status anzeigt
                updateUi();
            }
        });

        // So protokollieren Sie Ihre Log-Meldungen in adb logcat!
        Log.d("SilentModeToggle", "This is a test");
    }


    /**
     * Aktualisiert das UI-Bild, um das stummgeschaltete oder
     * normale Telefon anzuzeigen
     */
    private void updateUi() {
        // Die Ansicht phone_icon in unserem Layout finden. Wir wissen,
        // dass es sich um eine ImageView im Layout handelt, deshalb
        // grenzen wir das Ganze auf eine ImageView ein.
        ImageView imageView = (ImageView) findViewById(R.id.phone_icon);

        // Das phoneImage auf die ID des Bildes mit Klingel ein/aus
        // setzen. Zu finden in res/drawable-xxhdpi
        int phoneImage = RingerHelper.isPhoneSilent(audioManager)
                ? R.drawable.ringer_off
                : R.drawable.ringer_on;

        // die imageView auf das Bild in phoneImage setzen
        imageView.setImageResource(phoneImage);
    }

    /**
     * Immer wenn die Aktivitt fortgesetzt wird, sicherstellen, dass die
     * Schaltflchen aktualisert werden und den aktuellen Status des
     * Systems zeigen (weil der Benutzer vielleicht den Stummschaltungs-
     * modus gendert hat, whrend wir im Hintergrund waren.
     *
     * In http://d.android.com/reference/android/app/Activity.html
     * erfahren Sie mehr ber den Android Activity Lifecycle.
     */
    @Override
    protected void onResume() {
        super.onResume();

        // Unsere UI aktualisieren, falls sich etwas gendert hat.
        updateUi();
    }
}