package com.dummies.tasks.util;

import android.app.AlarmManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;

import com.dummies.tasks.provider.TaskProvider;
import com.dummies.tasks.receiver.OnAlarmReceiver;

import java.util.Calendar;

/**
 * Eine Helfer-Klasse, die mit dem AlarmManager Erinnerungen einstellen kann
 */
public class ReminderManager {

    // Die Klasse soll nicht instantiiert werden, deshalb den Konstruktor
    // privat machen
    private ReminderManager() {}

    public static void setReminder(Context context, long taskId,
                                   String title, Calendar when) {

        AlarmManager alarmManager = (AlarmManager) context
                .getSystemService(Context.ALARM_SERVICE);

        // Einen Intent fr unseren OnAlarmReceiver erstellen,
        // der die Benachrichtigung anzeigt, wenn er
        // aufgerufen wird
        Intent i = new Intent(context, OnAlarmReceiver.class);
        i.putExtra(TaskProvider.COLUMN_TASKID, taskId);
        i.putExtra(TaskProvider.COLUMN_TITLE, title);

        // Den PendingIntent erstellen, der den obigen Intent
        // aufnimmt. Alle Intents, die im AlarmManager verwendet
        // werden, mssen in einen PendingIntent eingehllt werden,
        // um dem AlarmManager die "Berechtigung zu erteilen",
        // einen Callback in unsere App durchzufhren.
        PendingIntent pi = PendingIntent.getBroadcast(context, 0, i,
                PendingIntent.FLAG_ONE_SHOT);

        // Den Alarm einstellen
        alarmManager.set(AlarmManager.RTC_WAKEUP, when.getTimeInMillis(),
                pi);
    }
}
