package com.dummies.tasks.tablet.activity;

import android.app.Activity;
import android.app.Fragment;
import android.app.FragmentManager;
import android.app.FragmentTransaction;
import android.os.Bundle;
import android.widget.Toolbar;

import com.dummies.tasks.R;
import com.dummies.tasks.fragment.TaskEditFragment;
import com.dummies.tasks.interfaces.OnEditFinished;
import com.dummies.tasks.interfaces.OnEditTask;

/**
 * Unsere Aktivitt mit Erinnerungsliste mit Bearbeitung fr Tablets
 */
public class TaskListAndEditorActivity extends Activity
        implements OnEditTask, OnEditFinished
{

    /**
     * Wird beim ersten Erstellen der Aktivitt aufgerufen.
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // Layout aus einer XML-Datei einrichten
        setContentView(R.layout.activity_task_list_and_editor);

        // Android mitteilen, dass R.id.toolbar unsere ActionBar ist
        setActionBar((Toolbar) findViewById(R.id.toolbar));
    }

    /**
     * Wird aufgerufen, wenn der Benutzer anfordert, eine Task
     * zu bearbeiten oder einzutragen.
     */
    @Override
    public void editTask(long id) {
        // Das Fragment fr die Task-ID erstellen
        TaskEditFragment fragment = TaskEditFragment.newInstance(id);

        // Der Aktivitt das Fragment hinzufgen. Wenn es bereits eines gibt,
        // (z. B. der Benutzer klickt auf eine andere Task), dieses ersetzen
        FragmentTransaction ft = getFragmentManager()
                .beginTransaction();
        ft.replace(R.id.edit_container, fragment,
                TaskEditFragment.DEFAULT_FRAGMENT_TAG);

        // Diese nderung dem Backstack hinzufgen. Wenn der Benutzer also
        // auf die Schaltflche "Zurck" klickt, holen wir diesen Editor
        // vom Stack. Andernfalls wird die gesamte Aktivitt geschlossen,
        // wenn der Benutzer auf die Schaltflche "Zurck" klickt, was
        // strend und unerwartet ist.
        ft.addToBackStack(null);

        // Wir machen es so!
        ft.commit();
    }

    /**
     * Wird aufgerufen, wenn der Benutzer mit der Bearbeitung einer Task fertig ist.
     */
    @Override
    public void finishEditingTask() {
        FragmentManager fm = getFragmentManager();
        FragmentTransaction transaction = fm
                .beginTransaction();

        // Das bearbeitete Fragment finden und aus der Aktivitt entfernen.
        Fragment previousFragment = fm
                .findFragmentByTag(
                    TaskEditFragment.DEFAULT_FRAGMENT_TAG);
        transaction.remove(previousFragment);

        transaction.commit();
    }

}
