package com.dummies.tasks;

import android.annotation.SuppressLint;
import android.content.Context;
import android.support.wearable.view.WearableListView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.google.android.gms.wearable.DataItem;
import com.google.android.gms.wearable.DataMap;
import com.google.android.gms.wearable.DataMapItem;

import java.util.List;

/**
 * Ein WearableListAdapter, der unsere Task-Element in einer
 * Liste anzeigen kann.
 */
public class WearableTaskListAdapter
    extends WearableListView.Adapter
{

    // Muss mit dem in der Phone-App verwendeten Namen bereinstimmen
    static final String COLUMN_TITLE = "title";

    // Aktuelle Liste der dataItems
    List<DataItem> dataItems;

    // Layout-Erweiterung
    LayoutInflater inflater;

    public WearableTaskListAdapter(Context context) {
        // Einen LayoutInflater aus dem aktuellen Kontext ermitteln
        inflater = LayoutInflater.from(context);
    }

    /**
     * Einen ViewHolder erstellen, der eine Referenz auf die Views
     * aufnimmt, die wir fr jedes neue Element in der Liste
     * aktualisieren mssen.
     */
    @SuppressLint("InflateParams")
    @Override
    public WearableListView.ViewHolder onCreateViewHolder(
        ViewGroup viewGroup, int i) {

        // EInen neuen ViewHolder zurckgeben (siehe unten, Klasse ViewHolder).
        // Jede Ansicht in unserer Liste verwendet das Layout item_task.xml.
        return new ViewHolder(
            inflater.inflate(R.layout.item_task, null));
    }

    /**
     * Die Ansichten im ViewHolder mit der Information in Element
     * an Position i aktualisieren.
     */
    @Override
    public void onBindViewHolder(
        WearableListView.ViewHolder viewHolder, int i) {

        // Das DataItem fr das Element in Position i finden.
        DataItem dataItem = dataItems.get(i);

        // Die ursprngliche DataMap fr dieses Element wiederherstellen
        DataMap map
            = DataMapItem.fromDataItem(dataItem).getDataMap();

        // Den Titeltext basierend auf dem COLUMN_TITLE in der
        // DataMap erstellen
        ((ViewHolder) viewHolder).titleView.setText(
            map.getString(COLUMN_TITLE)
        );
    }

    @Override
    public int getItemCount() {
        return dataItems != null ? dataItems.size() : 0;
    }

    /**
     * Aktualisieren der Elemente in der Liste und benachrichtigen
     * der Listener (insbesondere ListView), dass sich die Daten im
     * Adapter gendert haben.
     */
    public void setResults(List<DataItem> dataItems) {
        this.dataItems = dataItems;
        notifyDataSetChanged();
    }


    /**
     * Einfacher ViewHolder, der nur die titleView fr unser Listenelement
     * enthlt.
     */
    static class ViewHolder extends WearableListView.ViewHolder {
        TextView titleView;

        public ViewHolder(View itemView) {
            super(itemView);
            titleView = (TextView) itemView.findViewById(R.id.title);
        }

    }
}
