package com.dummies.tasks;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.common.api.ResultCallback;
import com.google.android.gms.wearable.Node;
import com.google.android.gms.wearable.Wearable;

import java.nio.charset.Charset;

import static com.google.android.gms.wearable.NodeApi
    .GetConnectedNodesResult;

public class TakeVoiceNote extends Activity
    implements GoogleApiClient.ConnectionCallbacks, GoogleApiClient.OnConnectionFailedListener
{
    private static final String PATH_ADD_TASK = "/addTask";

    GoogleApiClient googleApiClient;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        googleApiClient = new GoogleApiClient.Builder(this)
            .addApi(Wearable.API)
            .addConnectionCallbacks(this)
            .addOnConnectionFailedListener(this)
            .build();
    }

    @Override
    protected void onStart() {
        super.onStart();

        // Verbindung mit dem GoogleApiClient.  Wenn wir verbunden sind,
        // erhalten wir einen Callback fr onConnected().
        googleApiClient.connect();
    }

    @Override
    protected void onStop() {
        super.onStop();

        // Wir sind fertig, deshalb vom GoogleApiClient trennen.
        googleApiClient.disconnect();
    }

    @Override
    public void onConnected(Bundle bundle) {
        Log.d("TakeVoiceNote", "onConnected");
        
        // Der Spracherkennungstext wird ber den Intent bergeben
        String voiceNote = getIntent().getStringExtra(
            Intent.EXTRA_TEXT);

        final byte[] voiceNoteBytes =
            voiceNote.getBytes(Charset.forName("utf-8"));


        Wearable.NodeApi.getConnectedNodes(googleApiClient)
            .setResultCallback(
                new ResultCallback<GetConnectedNodesResult>() {
                    @Override
                    public void onResult(GetConnectedNodesResult nodes) {
                        for (Node node : nodes.getNodes()) {
                            
                            // Dem Telefon eine Nachricht senden, um
                            // anzufordern, dass es der Datenbank die
                            // Task hinzufgt
                            Wearable.MessageApi.sendMessage(
                                googleApiClient,
                                node.getId(),
                                PATH_ADD_TASK,
                                voiceNoteBytes
                                );
                            
                        }
                        
                        finish();
                    }
                }
            );
    }

    @Override
    public void onConnectionSuspended(int i) {
        // Einfach eine Nachricht protokollieren. Wir mssen nichts tun,
        // auer eine Nachricht zu protokollieren, weil wir auf diese
        // Weise einfacher Fehler finden knnen.
        Log.d("TakeVoiceNote", "onConnectionSuspended");
    }

    @Override
    public void onConnectionFailed(ConnectionResult connectionResult) {
        / Einfach eine Nachricht protokollieren. Wir mssen nichts tun,
        // auer eine Nachricht zu protokollieren, weil wir auf diese
        // Weise einfacher Fehler finden knnen.
        Log.d("TakeVoiceNote", "onConnectionFailed");
    }

}
